@@ -47,6 +47,7 @@
 /* Does the host have /var/run directory? */
 #undef HAVE_VAR_RUN
 
+
 /* Special OS defines */
 #undef SUN3
 #undef SUN4
@@ -73,6 +74,7 @@
 #undef HAVE_SYS_ACL_H
 #undef NOTBROKEN
 #undef NT
+#undef DARWIN
 
 /* SVR4 header stuff */
 
@@ -85,8 +87,7 @@
 
 /* LOCK and LOG directories */
 
-#undef LOCKFILEDIR
-#undef LOGFILEDIR
+#undef WORKDIR
 
 /* Special CFEngine symbols */
 
