@@ -492,6 +492,7 @@ for (ip = SCHEDULE; ip != NULL; ip = ip->next)
    }
 
 DeleteItemList(VHEAP);
+VHEAP = NULL; 
 GetNameInfo();
 return false;
 }
@@ -767,13 +768,15 @@ int number;
 }
 
 
-void RecursiveTidySpecialArea(name,tp,maxrecurse)
+int RecursiveTidySpecialArea(name,tp,maxrecurse,sb)
 
 char *name;
 struct Tidy *tp;
 int maxrecurse;
+struct stat *sb;
 
 {
+ return true;
 }
 
 
