@@ -898,13 +898,14 @@ exit(1);
 }
 
 
-void RecursiveTidySpecialArea(name,tp,maxrecurse)
+int RecursiveTidySpecialArea(name,tp,maxrecurse,sb)
 
 char *name;
 struct Tidy *tp;
 int maxrecurse;
-
+struct stat *sb;
 {
+ return true;
 }
 
 int CompareMD5Net(file1,file2,ip)
