@@ -35,7 +35,7 @@
 
 /*******************************************************************/
 
-FixCompressedArrayValue(i,value,start)
+int FixCompressedArrayValue(i,value,start)
 
 int i;
 char *value;
@@ -78,7 +78,7 @@ return true;
 
 /*******************************************************************/
 
-DeleteCompressedArray(start)
+void DeleteCompressedArray(start)
 
 struct CompressedArray *start;
 
@@ -99,7 +99,7 @@ if (start != NULL)
 
 /*******************************************************************/
 
-CompressedArrayElementExists(start,key)
+int CompressedArrayElementExists(start,key)
 
 struct CompressedArray *start;
 int key;
