@@ -47,13 +47,13 @@ char *psopts;
 { FILE *pp;
   char pscomm[maxlinksize];
 
-sprintf(pscomm,"%s %s",VPSCOMM[VSYSTEMHARDCLASS],psopts);
+snprintf(pscomm,maxlinksize,"%s %s",VPSCOMM[VSYSTEMHARDCLASS],psopts);
 
 Verbose("%s: Running process command %s\n",VPREFIX,pscomm); 
   
 if ((pp = cfpopen(pscomm,"r")) == NULL)
    {
-   sprintf(OUTPUT,"Couldn't open the process list with command %s\n",pscomm);
+   snprintf(OUTPUT,bufsize*2,"Couldn't open the process list with command %s\n",pscomm);
    CfLog(cferror,OUTPUT,"popen");
    return false;
    }
@@ -76,12 +76,9 @@ void DoProcessCheck(pp,procdata)
 struct Process *pp;
 struct Item *procdata;
 
-{ char line[bufsize], *sp;
-  int match, pid=-1, ret, matches=0,dosignals=true;
-  int BreakPipe();
-  FILE *pipe;
+{ char line[bufsize];
+  int matches=0,dosignals=true;
   mode_t maskval;
-  size_t nmatch = 1;
   struct Item *killlist = NULL;
   
 matches = FindMatches(pp,procdata,&killlist); 
@@ -103,7 +100,7 @@ if (pp->matches >= 0)
       {
       case '=': if (matches != (int)pp->matches)
                    {
-	    	   sprintf(OUTPUT,"%d processes matched %s (should be %d)\n",matches,pp->expr,pp->matches);
+	    	   snprintf(OUTPUT,bufsize*2,"%d processes matched %s (should be %d)\n",matches,pp->expr,pp->matches);
 		   CfLog(cferror,OUTPUT,"");
 		   if (pp->action == 'm')
 		      {
@@ -114,7 +111,7 @@ if (pp->matches >= 0)
 		
       case '>': if (matches < (int)pp->matches)
 	           {
-	    	   sprintf(OUTPUT,"%d processes matched %s (should be >=%d)\n",matches,pp->expr,pp->matches);
+	    	   snprintf(OUTPUT,bufsize*2,"%d processes matched %s (should be >=%d)\n",matches,pp->expr,pp->matches);
 		   CfLog(cferror,OUTPUT,"");
 		   if (pp->action == 'm')
 		      {
@@ -125,7 +122,7 @@ if (pp->matches >= 0)
 		
       case '<': if (matches > (int)pp->matches)
 	           {
-	    	   sprintf(OUTPUT,"%d processes matched %s (should be <=%d)\n",matches,pp->expr,pp->matches);
+	    	   snprintf(OUTPUT,bufsize*2,"%d processes matched %s (should be <=%d)\n",matches,pp->expr,pp->matches);
 		   CfLog(cferror,OUTPUT,"");
 		   if (pp->action == 'm')
 		      {
@@ -156,7 +153,7 @@ if (strlen(pp->restart) != 0)
       return;
       }
    
-   sprintf(OUTPUT,"Executing shell command: %s\n",pp->restart);
+   snprintf(OUTPUT,bufsize*2,"Executing shell command: %s\n",pp->restart);
    CfLog(cfinform,OUTPUT,"");
 
    if (DONTDO)
@@ -169,7 +166,7 @@ if (strlen(pp->restart) != 0)
    
    if (pp->umask == 0)
       {
-      sprintf(OUTPUT,"Programming %s running with umask 0! Use umask= to set\n",pp->restart);
+      snprintf(OUTPUT,bufsize*2,"Programming %s running with umask 0! Use umask= to set\n",pp->restart);
       CfLog(cfsilent,OUTPUT,"");
       }
    
@@ -177,7 +174,7 @@ if (strlen(pp->restart) != 0)
       {
       if ((PIPE = cfpopen_shsetuid(pp->restart,"r",pp->uid,pp->gid,pp->chdir,pp->chroot)) == NULL)
 	 {
-	 sprintf(OUTPUT,"Process restart execution failed on %s\n",pp->restart);
+	 snprintf(OUTPUT,bufsize*2,"Process restart execution failed on %s\n",pp->restart);
 	 CfLog(cferror,OUTPUT,"popen");
 	 return;
 	 }
@@ -186,7 +183,7 @@ if (strlen(pp->restart) != 0)
       {
       if ((PIPE = cfpopensetuid(pp->restart,"r",pp->uid,pp->gid,pp->chdir,pp->chroot)) == NULL)
 	 {
-	 sprintf(OUTPUT,"Process restart execution failed on %s\n",pp->restart);
+	 snprintf(OUTPUT,bufsize*2,"Process restart execution failed on %s\n",pp->restart);
 	 CfLog(cferror,OUTPUT,"popen");
 	 return;
 	 }
@@ -218,11 +215,10 @@ if (strlen(pp->restart) != 0)
 
       if (strstr(line,"cfd: start") || DEADLOCK)
 	 {
-         printf("");
 	 break;
 	 }
       
-      sprintf(OUTPUT,"Restart: %s",line);
+      snprintf(OUTPUT,bufsize*2,"Restart: %s",line);
       CfLog(cfinform,OUTPUT,"");
       }
 
@@ -235,7 +231,7 @@ if (strlen(pp->restart) != 0)
       cfpclose(PIPE);
       }
    
-   sprintf(OUTPUT,"(Done with %s)\n",pp->restart);
+   snprintf(OUTPUT,bufsize*2,"(Done with %s)\n",pp->restart);
    CfLog(cfinform,OUTPUT,"");
    umask(maskval);
    }
@@ -249,8 +245,8 @@ struct Process *pp;
 struct Item *procdata,**killlist;
 
 { struct Item *ip, *ip2;
-  char *sp, *err,saveuid[16];
-  int match, pid=-1, ret, matches=0, got, i;
+  char *sp,saveuid[16];
+  int pid=-1, ret, matches=0, got, i;
   regex_t rx,rxcache;
   regmatch_t pmatch;
   pid_t cfengine_pid = getpid();
@@ -260,12 +256,8 @@ struct Item *procdata,**killlist;
   
 Debug2("Looking for process %s\n",pp->expr);
 
-if (regcomp(&rxcache,pp->expr,REG_EXTENDED) != 0)
+if (CfRegcomp(&rxcache,pp->expr,REG_EXTENDED) != 0)
    {
-   sprintf(OUTPUT,"Regular expression error for process pattern %s\n",pp->expr);
-   CfLog(cferror,OUTPUT,"");
-   sprintf(OUTPUT,"%s\n",err);
-   CfLog(cferror,OUTPUT,"");
    return 0;
    }
 
@@ -316,17 +308,19 @@ for (ip = procdata; ip != NULL; ip=ip->next)
 	 {
 	 continue;
 	 }
+
+      Debug("Matched proc[%s]\n",ip->name);
       
       for (sp = ip->name; *sp != '\0'; sp++) /* if first field contains alpha, skip */
 	 {
 	 while (true)
 	    {
-	    while (!isdigit(*sp) && (*sp != '\0'))
+	    while (!isdigit((int)*sp) && (*sp != '\0'))
 	       {
 	       sp++;
 	       }
 	    
-	    if ((sp > ip->name) && isalnum(*(sp-1))) /* Username contains number*/
+	    if ((sp > ip->name) && isalnum((int)*(sp-1))) /* Username contains number*/
 	       {
 	       sp++;
 	       }
@@ -346,7 +340,7 @@ for (ip = procdata; ip != NULL; ip=ip->next)
       
       if (pid == -1)
 	 {
-	 sprintf(OUTPUT,"Unable to extract pid while looking for %s\n",pp->expr);
+	 snprintf(OUTPUT,bufsize*2,"Unable to extract pid while looking for %s\n",pp->expr);
 	 CfLog(cfverbose,OUTPUT,"");
 	 continue;
 	 }
@@ -368,16 +362,16 @@ for (ip = procdata; ip != NULL; ip=ip->next)
       
       if (pp->action == 'w')
 	 {
-	 sprintf(OUTPUT,"Process alert: %s\n",procdata->name);
+	 snprintf(OUTPUT,bufsize*2,"Process alert: %s\n",procdata->name);
 	 CfLog(cferror,OUTPUT,"");	 
-	 sprintf(OUTPUT,"Process alert: %s\n",ip->name);
+	 snprintf(OUTPUT,bufsize*2,"Process alert: %s\n",ip->name);
 	 CfLog(cferror,OUTPUT,"");
 	 continue;
 	 }
       
       if (pp->signal != cfnosignal)
 	 {
-	 sprintf(OUTPUT,"Signalling process %d (%s) with %s\n",pid,pp->expr,SIGNALS[pp->signal]);
+	 snprintf(OUTPUT,bufsize*2,"Signalling process %d (%s) with %s\n",pid,pp->expr,SIGNALS[pp->signal]);
 	 CfLog(cfinform,OUTPUT,"");
 	 
 	 if (!DONTDO)
@@ -397,7 +391,7 @@ for (ip = procdata; ip != NULL; ip=ip->next)
 	       {
 	       if ((ret = kill((pid_t)pid,pp->signal)) < 0)
 		  {
-		  sprintf(OUTPUT,"Couldn't send signal to pid %d\n",pid);
+		  snprintf(OUTPUT,bufsize*2,"Couldn't send signal to pid %d\n",pid);
 		  CfLog(cfverbose,OUTPUT,"kill");
 		  
 		  continue;
@@ -405,7 +399,7 @@ for (ip = procdata; ip != NULL; ip=ip->next)
 	       
 	       if ((pp->signal == cfkill || pp->signal == cfterm) && ret >= 0)
 		  {
-		  sprintf(OUTPUT,"Killed: %s\n",ip->name);
+		  snprintf(OUTPUT,bufsize*2,"Killed: %s\n",ip->name);
 		  CfLog(cfinform,OUTPUT,"");
 		  }
 	       }
@@ -466,7 +460,7 @@ for (ip = list; ip != NULL; ip=ip->next)
       {
       if ((ret = kill((pid_t)pid,pp->signal)) < 0)
 	 {
-	 sprintf(OUTPUT,"Couldn't send signal to pid %d\n",pid);
+	 snprintf(OUTPUT,bufsize*2,"Couldn't send signal to pid %d\n",pid);
 	 CfLog(cfverbose,OUTPUT,"kill");
 	 
 	 return;
@@ -474,7 +468,7 @@ for (ip = list; ip != NULL; ip=ip->next)
    
       if ((pp->signal == cfkill || pp->signal == cfterm) && ret >= 0)
 	 {
-	 sprintf(OUTPUT,"Killed: %s\n",ip->name);
+	 snprintf(OUTPUT,bufsize*2,"Killed: %s\n",ip->name);
 	 CfLog(cfinform,OUTPUT,"");
 	 }
       }
